#-----------------------------------------------------------------------------
# Vehicle steering script, version 4, with banking and "diving"
# by D. Drumright (Ockham)
# 03/29/04
#
#-----------------------------------------------------------------------------
from Tkinter import *
import poser
scene = poser.Scene()
import poser
import math
global OldY
OldY = 0.0  # Holder for deltas

scene = poser.Scene()

# Find the Platform and Vehicle.
Platform = scene.CurrentActor()
VL = Platform.Children()
try:
    Vehicle = VL[0]
except:
    raise "Vehicle is not parented to Platform."

#-----------------------------------------------------------------------------
def Steer(thisframe,BankConstant):
    global OldY
    # Apply the Yrot to the Platform, and the other two rots to the Vehicle.
    scene.SetFrame(thisframe) # Put everything into first position
    PresentOrigin = Platform.WorldDisplacement() # Where is the object in first position?
    aheadframe = thisframe + 5 # second position is further ahead
    if aheadframe < scene.NumFrames() : # protect against end of list
       scene.SetFrame(aheadframe) # Put everything into the new position
       AheadOrigin = Platform.WorldDisplacement() # Where is the Vehicle in new position?
       Xdist = AheadOrigin[0] - PresentOrigin[0]  # Figure X distance old to new
       Zdist = AheadOrigin[2] - PresentOrigin[2]  # Figure Z distance old to new
       RawY = math.atan2(-Zdist,Xdist)  # Angle of vector from old to new

       scene.SetFrame(thisframe)  # Come back to where we really are
       #Now set the Y rotation of the Platform to anticipate this direction

       NewY = RawY * 180.0 / math.pi # Turn to degrees

       alterParm = Platform.ParameterByCode(poser.kParmCodeYROT)
       alterParm.SetValue(NewY+90.0) # Set Y rotation in degrees

       # Now set the banking (roll) of the Vehicle itself.
       DeltaY = NewY - OldY
       Bank = BankConstant * math.sin(DeltaY)
       zParm = Vehicle.ParameterByCode(poser.kParmCodeZROT)
       zParm.SetValue(Bank)

       OldY = NewY  # Hold for next


#-----------------------------------------------------------------------------

def Dive(thisframe,BankConstant):

    scene.SetFrame(thisframe) # Put everything into first position --
    PresentOrigin = Platform.WorldDisplacement() # Where is the Platform in first position?
    aheadframe = thisframe + 5 # second position is further ahead
    if aheadframe < scene.NumFrames() : # protect against end of list
       scene.SetFrame(aheadframe) # Put everything into the new position
       AheadOrigin = Platform.WorldDisplacement() # Where is the Platform in new position?
       scene.SetFrame(thisframe)  # Come back to where we really are

       # Figure diffs for each:
       Xdist = float(AheadOrigin[0] - PresentOrigin[0])
       Ydist = float(AheadOrigin[1] - PresentOrigin[1])
       Zdist = float(AheadOrigin[2] - PresentOrigin[2])

       # Net horizontal distance:
       HorDist = math.hypot(Xdist,Zdist)

       # Save from divide by zero....
       if HorDist == 0.0:
           DeltaPitch = 0.0
       else:
           # Slope is arctangent of Ydist / Hordist.
           DeltaPitch = math.atan2(-Ydist,HorDist)

       FinalDelta = DeltaPitch * BankConstant * 180.0 / math.pi
       xParm = Vehicle.ParameterByCode(poser.kParmCodeXROT)
       xParm.SetValue(FinalDelta)
#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Bank-dive")

        self.BankLabel=Label(self.master, text="Bank:")
        self.BankLabel.grid(row=0,column=0,sticky=W)
        self.BankSlider=Scale(self.master, from_ = -50.0, to = 50.0,orient=HORIZONTAL,resolution=0.5)
        self.BankSlider.grid(row=0, column=1)

        self.DiveLabel=Label(self.master, text="Dive:")
        self.DiveLabel.grid(row=1,column=0,sticky=W)
        self.DiveSlider=Scale(self.master, from_ = -50.0, to = 50.0,orient=HORIZONTAL,resolution=0.5)
        self.DiveSlider.grid(row=1, column=1)


        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=3, column=2)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents(1)
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        # Get the steerval and diveval from sliders
        BankVal = self.BankSlider.get()
        DiveVal = self.DiveSlider.get()
        # First apply the Banking, all the way through the frames.
        for i in range(scene.NumFrames()):
            Steer(i, BankVal) # Set the number to control strength of Banking

        # Now start from the beginning again and apply the Dive actions.
        for i in range(scene.NumFrames()):
            Dive(i, DiveVal) # Set the number to control strength of Diving

        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()


#-----------------------------------------------------------------------------



#-----------------------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------
